/*
  Servo360driver.c  ver 20171111b
                    This is the test harness / demo for the servo360a library
       Based on a Spin/PASM driver by Tom Crawford
     
     Group sections in this order:
      Demo global declarations
      Demo functions
      main()
      
*/
#include "simpletools.h"                      // Include simple tools
#include "stdint.h"
#include "servo360a.h"

//  ======================================================
//       DEMO defines, variables, function prototypes
//  ======================================================
// **** defines for demo
#define CP0 12
#define CP1 13
#define CP2 14
#define CP3 0
#define FP0 8
#define FP1 9
#define FP2 10 
#define FP3 0

// **** Declare demo globals
// ***  Servo pins control and feedback, if both 0 -- no servo

 int16_t CPIN[] = {CP0, CP1, CP2, CP3};
 int16_t FPIN[] = {FP0, FP1, FP2, FP3};
volatile int8_t  deltaMax, deltaMin, Cog, DisplayServo, currentservo, testgain;  
volatile int16_t ordered, actual, rcount, theservo, Cmnded[4];    // short sub ordered for commanded
int random;


// **** Demo function prototypes
void MySet(short, short);       //  servonum, degrees
short WaitSet(short);            //  servonum,  returns position in deg
void showangles();

//  ======================================================
//        Start DEMO FUNCTIONS 
//  ======================================================
//  ******************************************************
//         Set ordered position
//  ******************************************************
void MySet(short servonum, short degrees)
{
  SetPos(servonum, degrees);
}   

//  ******************************************************
//    Wait until servo has completed ordered movement     
//  ******************************************************
short WaitSet(short servonum)
{
  while(CheckDone(servonum));
  pause(10);
  return GetPos(servonum);
}

//  ******************************************************
//    Print servo position in degrees     
//  ******************************************************
void showangles()
{
  short servonum;
  for(servonum = 0; servonum <= 2; servonum++)
    {
       print("  Servo%d %d  ",servonum, GetPos(servonum));
    }        
}
//  ******************************************************
//        END DEMO FUNCTIONS 
//  ******************************************************
//  ******************************************************

int main()                   // The DEMO
{
 start360(CPIN, FPIN); 
 print("cog360 = %d \n", cog360servo);
 pause(500);

// Parameters values set for all servos in start360() 
// Parameters[i] = (2<<24)|(18<<16)|(31<<8)|150;    // in spin: error-3, gain-12, deltamin-31, deltamax-150 

// ***** Change parameters for individual servos 
 SetMaxError(Servo0, 2);
//SetMaxError(Servo1, 4); 
//SetMaxError(Servo2, 4);
 pause(10);
 
 SetGain(Servo0, 16); // 22
 SetGain(Servo1, 12); // 24
 //SetGain(Servo2, 20);
  pause(10); 
 
 SetDeltaMin(Servo0, 29);  //29
 SetDeltaMin(Servo1, 29);
 SetDeltaMin(Servo2, 31);
 pause(10);
//  ***** End parameter set *****

//  ***** command servos to move *****
 Width(Servo0, 1450);   // start servos rotating 1280 - 1720
 Width(Servo1, 1650);
 Width(Servo2, 1450);
 Width(Servo3, 1550);
 
 for(int i = 0; i < 50; i++)
  {
    showangles();
    print("\n");
    pause(10);
  }       
 for(theservo = 0; theservo <= 2; theservo++) Width(theservo,1500);   // stop rotation
 pause(4000);

// rotate servo0 by one degree
  int degree = 1;
  MySet(0, degree);
  actual =  WaitSet(0);
  //pause(2000);
  //actual = GetPos(0); 
  print("\nCommanded: %3d  Actual: %3d", degree, actual);   
  for(int i = 2; i<=90; i++)
    {
      MySet(0, i);
      //pause(2000);
      actual =  WaitSet(0);
      //actual = GetPos(0);
      print("\nCommanded: %3d  Actual: %3d  Error: %3d", i, actual, i-actual);
      pause(200);
    } 
 
 
 for(theservo = 0; theservo <= 2; theservo++)   // full scale changes
  {
    print("servo %d  ",theservo);
    MySet(theservo, 180);
   // pause(1000);
    //actual = GetPos(theservo);
    actual = WaitSet(theservo);
    print("%d  ",actual);
    MySet(theservo, 0);
   // pause(1000);
    actual = WaitSet(theservo);
    //actual = GetPos(theservo);
    print("%d  ",actual);    
    MySet(theservo, 358);
   // pause(1500);
    actual = WaitSet(theservo);    
    //actual = GetPos(theservo);
    print("%d  ",actual);    
    MySet(theservo, 0);
   // pause(1200);
    actual = WaitSet(theservo);
    //actual = GetPos(theservo);
    print("%d  \n",actual);
  }                             // end full scale changes

  srand(CNT);
  random = rand();
  rcount = 0;
  while(rcount < 100)
    {
      for(theservo = 0; theservo <= 2; theservo++)   // 
      {
       if(CheckDone(theservo) == 0)
        {
         print("\n%d  Commanded: %3d  Actual: %3d", theservo, Cmnded[theservo], GetPos(theservo));
         print("  Error: %2d", Cmnded[theservo] - GetPos(theservo));
         
         random = rand();
         ordered = abs(random) % 360;
         MySet(theservo, ordered);
         pause(200);
         Cmnded[theservo] = ordered;
         print("   %2d", rcount++);
        }   // end if      
      }     // end for 
    }       // end while
}   // end main

