
#include "servo360a.h"

//  **** Declare DRIVER globals in servo360a.h

//  ======================================================
//            START DRIVER FUNCTIONS
//  ======================================================

//  ***************************
//      Initialize
//  ***************************
void start360(short *CPIN, short *FPIN)     // pin numbers from define in header
{
  
 ServoPins[0] = CPIN[0] << 8 | FPIN[0];
 ServoPins[1] = CPIN[1] << 8 | FPIN[1];
 ServoPins[2] = CPIN[2] << 8 | FPIN[2];
 ServoPins[3] = CPIN[3] << 8 | FPIN[3];

 for( int i = 0; i <= 3; i++)
   {
     if (ServoPins[i] != 0)
      {
        mode[i] = 0;
        PulseWidth[i] = 1500;                         
        Parameters[i] = (2<<24)|(18<<16)|(31<<8)|150;    // error-3, gain-12, deltamin-31, deltamax-150
      } // end if
    }   // end for i
 MyParm[0] = PulseWidth;
 MyParm[1] = mode; 
 MyParm[2] = theta;
 MyParm[3] = commanded; 
 MyParm[4] = Complete;        
 MyParm[5] = CLKFREQ / 1000000;
 MyParm[6] = ServoPins; 
 MyParm[7] = Context;
 MyParm[8] = Parameters;

 extern int binary_FourPly_dat_start[]; 
 cog360servo = cognew((void*)binary_FourPly_dat_start, MyParm); 
 pause(500);          
 }
 
//  ******************************************************
//        Order servo to position - degrees 
//  ******************************************************
void SetPos(short servonum, short degrees)
{
  commanded[servonum] = degrees;
  Complete[servonum] = 1;                  //we are not complete yet
  mode[servonum] = 1;                      //set position mode
 }
 
//  ******************************************************
//      Make servo receive pulses of spec'd width
//  ****************************************************** 
void Width(short servonum, short Usec)
{
  mode[servonum]= 0;                //  CPW mode
  PulseWidth[servonum] = Usec;      // save pulse width in usec
 }
 
//  ******************************************************
//      Get the current position of the servo
//  ****************************************************** 
short GetPos(short servonum)
{
  int16_t xxx = theta[servonum];
  return xxx;
  //return theta[servonum];    // can't return an array element
}  

//  ******************************************************
//     Check if servo is at commanded position
//     Returns Complete[servonum]
//       =0 if in commanded position +/- error for 200ms
//      !=0 if not in position 
//  ******************************************************  
int8_t CheckDone(short servonum)
{
  int8_t xxx = Complete[servonum];
  return xxx;
  //return Complete[servonum];    // can't return an array element
}  

//  ******************************************************
//    Define the maximum allowable error for spec'd servo
//      accepterror = 1 degree greater than acceptable error
//  ****************************************************** 
void SetMaxError(short servonum, int8_t accepterror)
{
  Parameters[servonum] = Parameters[servonum] & 0x00FFFFFF;
  Parameters[servonum] = Parameters[servonum] | (accepterror << 24);  
}

//  ******************************************************
//   Define the gain for the specified servo. 
//    typical 5 to 15, higher results in more overshoot   
//  ******************************************************
void SetGain(short servonum, int8_t gainP)
{
  Parameters[servonum] = Parameters[servonum] & 0xFF00FFFF;  
  Parameters[servonum] = Parameters[servonum] | (gainP << 16);
}

//  ******************************************************
//   Specifies the minimum deviation from 1500 usec 
//    Use to keep servo from being stuck in dead zone
//    Typical values approx 30  
//  ******************************************************
void SetDeltaMin(short servonum, int8_t Dmin) 
{
  Parameters[servonum] = Parameters[servonum] & 0xFFFF00FF;  
  Parameters[servonum] = Parameters[servonum] | (Dmin << 8);  
}

//  ******************************************************
//    Specifies the maximum deviation from 1500 usec.
//      Typical value approx 150
//  ******************************************************
void SetDeltaMax(short servonum, int8_t Dmax)
{
  Parameters[servonum] = Parameters[servonum] & 0xFFFFFF00;  
  Parameters[servonum] = Parameters[servonum] | Dmax;   
}
//  ======================================================
//        END DRIVER FUNCTIONS 
//  ======================================================
